package enums

import (
	"bytes"
	"errors"
)

type MemberStatus int8

const (
	MemberStatusInvalid  MemberStatus = 0
	MemberStatusDisabled MemberStatus = 1
	MemberStatusLocked   MemberStatus = 2
	MemberStatusEnabled  MemberStatus = 99
)

func MemberStatusNameOf(value string) MemberStatus {
	switch value {
	case "INVALID":
		return MemberStatusInvalid
	case "DISABLED":
		return MemberStatusDisabled
	case "LOCKED":
		return MemberStatusLocked
	case "ENABLED":
		return MemberStatusEnabled
	}
	return MemberStatusInvalid
}

func (status MemberStatus) Name() string {
	switch status {
	case MemberStatusInvalid:
		return "INVALID"
	case MemberStatusDisabled:
		return "DISABLED"
	case MemberStatusLocked:
		return "LOCKED"
	case MemberStatusEnabled:
		return "ENABLED"
	}
	return "INVALID"
}

func (status MemberStatus) String() string {
	switch status {
	case MemberStatusInvalid:
		return "알 수 없음"
	case MemberStatusDisabled:
		return "사용중지"
	case MemberStatusLocked:
		return "사용일시중지"
	case MemberStatusEnabled:
		return "사용중"
	}
	return "알 수 없음"
}
func (status *MemberStatus) UnmarshalJSON(from []byte) error {

	quote := []byte("\"")
	quoteSize := len(quote)

	if len(from) < quoteSize*2 {
		return errors.New("invalid quote notation")
	}

	if !bytes.HasPrefix(from, quote) || !bytes.HasSuffix(from, quote) {
		return errors.New("invalid quote notation")
	}
	*status = MemberStatusNameOf(string(from[quoteSize:len(from)-quoteSize]))
	return nil
}

func (status MemberStatus) MarshalJSON() ([]byte, error) {
	var buffer bytes.Buffer
	buffer.WriteRune('"')
	buffer.WriteString(status.Name())
	buffer.WriteRune('"')
	return buffer.Bytes(), nil
}
