package db

import (
	"database/sql/driver"
	"fmt"
	"strconv"
)

// NullUint64 represents an uint64 that may be null.
// NullUint64 implements the Scanner interface so
// it can be used as a scan destination, similar to NullString.

// NullUint64 is a sql.Scanner for unsigned ints.
type NullUint64 struct {
	Uint64 uint64
	Valid  bool
}

func (n NullUint64) Value() (driver.Value, error) {
	if !n.Valid {
		return nil, nil
	}
	return n.Uint64, nil
}

// Scan implements the sql.Scanner interface.
func (n *NullUint64) Scan(src interface{}) error {
	if src == nil {
		n.Uint64, n.Valid = 0, false
		return nil
	}
	n.Valid = true
	s := asString(src)
	var err error
	n.Uint64, err = strconv.ParseUint(s, 10, 64)
	return err
}

func asString(src interface{}) string {
	switch v := src.(type) {
	case string:
		return v
	case []byte:
		return string(v)
	}
	return fmt.Sprintf("%v", src)
}
