package iface

import (
	"strconv"
	"time"
)

//turns a new initialized CounterMetric object.
func NewCounterMetric() *CounterMetric {
	ca := &CounterMetric{}
	ca.Inc = make(chan Tuple)
	ca.internalRequestsSum = 0
	ca.internalRequests = make(map[string]uint64, 0)
	ca.internalRequestCodes = make(map[string]uint64, 0)
	return ca
}

// StartTimer will call a forever loop in a goroutine to calculate
// metrics for measurements every d ticks. The parameter of this
// function should normally be 1 * time.Minute, if not it will expose
// unintuive JSON keys (requests_per_minute and
// request_sum_per_minute).
func (ca *CounterMetric) StartTimer(d time.Duration) {
	timer := time.Tick(d)
	go func() {
		for {
			select {
			case tup := <-ca.Inc:
				ca.internalRequestsSum++
				ca.internalRequests[tup.Path]++
				ca.internalRequestCodes[strconv.FormatInt(int64(tup.Code), 10)]++
			case <-timer:
				ca.reset()
			}
		}
	}()
}

// GetStats to fulfill aspects.Aspect interface, it returns the data
// that will be served as JSON.
func (ca *CounterMetric) GetStats() interface{} {
	return *ca
}

// Name to fulfill aspects.Aspect interface, it will return the name
// of the JSON object that will be served.
func (ca *CounterMetric) Name() string {
	return "Counter"
}

// InRoot to fulfill aspects.Aspect interface, it will return where to
// put the JSON object into the monitoring endpoint.
func (ca *CounterMetric) InRoot() bool {
	return false
}

func (ca *CounterMetric) reset() {
	ca.RequestsSum = ca.internalRequestsSum
	ca.Requests = ca.internalRequests
	ca.RequestCodes = ca.internalRequestCodes
	ca.internalRequestsSum = 0
	ca.internalRequests = make(map[string]uint64, ca.RequestsSum)
	ca.internalRequestCodes = make(map[string]uint64, len(ca.RequestCodes))
}
