package redis

import (
	"fmt"
	"time"

	"amuz.es/gogs/infra/api_skeleton/util"
	"github.com/garyburd/redigo/redis"
	"github.com/youtube/vitess/go/pools"
	"golang.org/x/net/context"
)

var (
	pool   *pools.ResourcePool = nil
	logger                     = util.NewLogger("redis")
)

type resourceConn struct {
	redis.Conn
}

func (r resourceConn) Close() {
	r.Conn.Close()
}

func InitRedis(config *util.RedisConfig) {
	connectStr := fmt.Sprintf("%s:%d",
		config.Host,
		config.Port)
	logger.Debugf("connect to redis://%s:%d/%d",
		config.Host,
		config.Port,
		config.DB)

	pool = pools.NewResourcePool(func() (pools.Resource, error) {
		c, err := redis.Dial("tcp", connectStr,
			redis.DialDatabase(config.DB),
			redis.DialPassword(config.Password),
		)
		return resourceConn{c}, err
	}, 10, 10, time.Minute)

	logger.Info("Redis connected")
}

func getConn() (conn resourceConn, err error) {
	ctx := context.TODO()
	var resource pools.Resource
	if resource, err = pool.Get(ctx); err == nil {
		res := resource.(resourceConn)
		conn = res
	}
	return
}

func putConn(conn resourceConn) {
	pool.Put(conn)
}

func CloseRedis() {
	if pool != nil {
		pool.Close()
		logger.Info("Redis closed")
	}
}
