package redis

import (
	"math/rand"
	"time"

	retry "github.com/giantswarm/retry-go"
	errs "github.com/pkg/errors"
)

type txError error

func newTxError(err error) error {
	return newTxErrorWithMessage("key had changed during this tx", err)
}

func newTxErrorWithMessage(message string, err error) error {
	var wrapped error
	if err == nil {
		wrapped = errs.New(message)
	} else {
		wrapped = errs.Wrap(err, message)
	}
	return wrapped
}

func retries(tx func() error) error {
	return retry.Do(tx,
		retry.RetryChecker(func(err error) (ret bool) {
			switch err.(type) {
			case txError:
				ret = true
			}
			return
		}),
		retry.Timeout(400*time.Millisecond),
		retry.AfterRetry(func(err error) {
			if err == nil {
				return
			}
			sleeps := 37 + rand.Intn(51)
			time.Sleep(time.Duration(sleeps) * time.Millisecond)
		}),
		retry.MaxTries(5),
	)
}
