package util

import (
	"io/ioutil"

	"gopkg.in/yaml.v2"
)

type LogConfig struct {
	FileName  string
	MaxSizeMb int
	MaxBackup int
	MaxDay    int
}

type DbConfig struct {
	Network  string
	Driver   string
	Host     string
	User     string
	Password string
	Port     uint16
	DB       string
	Args     string
}

type RedisConfig struct {
	Host       string
	Port       uint16
	Password   string
	DB         int
	MaxRetries int
	PoolSize   int
}

type config struct {
	Phase   string
	Bind    string
	Db      DbConfig
	Redis   RedisConfig
	Logging struct {
		Application LogConfig
		Access      LogConfig
	}
}

func (c config) Name() string {
	return name
}

func (c config) Version() string {
	return version
}

func (c config) BuildDate() string {
	return buildDate
}

const name = "Api_skeleton"

var (
	buildDate string
	version   string
	Config    = config{}
)

func LoadConfig(path string) error {
	var (
		source []byte
		err    error
	)
	if source, err = ioutil.ReadFile(path); err != nil {
		return err
	}

	if err = yaml.Unmarshal(source, &Config); err != nil {
		return err
	}

	return nil
}
