package iface

import (
	"bytes"
	"database/sql/driver"
	"encoding/hex"
	"errors"
	"fmt"
)

func (b Bytes16) Marshal() ([]byte, error) {
	return b[:], nil
}

func (b Bytes16) MarshalTo(buf []byte) (n int, err error) {
	copy(buf, b[:])
	return len(b), nil
}
func (b *Bytes16) Unmarshal(buf []byte) error {
	if len(buf) != Bytes16Length {
		return fmt.Errorf("invalid bytes16 (got %d bytes)", len(buf))
	}
	copy(b[:], buf)
	return nil
}

func (b *Bytes16) UnmarshalHex(buf []byte) (err error) {
	hexLen := hex.DecodedLen(len(buf))
	if hexLen != Bytes16Length {
		return fmt.Errorf("invalid bytes16 (got %d bytes)", len(buf))

	}
	_, err = hex.Decode(b[:], buf)
	return
}

func (b Bytes16) MarshalHex() ([]byte, error) {
	hexLen := hex.EncodedLen(Bytes16Length)
	hexBuf := make([]byte,hexLen)
	if encodedLen := hex.Encode(hexBuf, b[:]); encodedLen != hexLen {
		return nil, fmt.Errorf("invalid bytes16 length: %d", len(hexBuf))
	}
	return hexBuf, nil
}

func (b Bytes16) MarshalJSON() (encoded []byte, err error) {
	encoded, err = b.MarshalHex()
	if err != nil {
		return
	}
	var buffer bytes.Buffer
	buffer.WriteRune('"')
	buffer.Write(encoded)
	buffer.WriteRune('"')
	return buffer.Bytes(), nil
}
func (b *Bytes16) UnmarshalJSON(from []byte) error {

	quote := []byte("\"")
	quoteSize := len(quote)

	if len(from) < quoteSize*2 {
		return errors.New("invalid quote notation")
	}

	if !bytes.HasPrefix(from, quote) || !bytes.HasSuffix(from, quote) {
		return errors.New("invalid quote notation")
	}
	return b.UnmarshalHex(from[quoteSize:len(from)-quoteSize])
}

func (b Bytes16) Compare(other Bytes16) int {
	return bytes.Compare(b[:], other[:])
}

func (b Bytes16) Equal(other Bytes16) bool {
	return b.Compare(other) == 0
}

func (b *Bytes16) Size() int {
	if b == nil {
		return 0
	}
	return Bytes16Length
}

// Scan implements the Scanner interface.
func (b *Bytes16) Scan(src interface{}) error {
	switch src.(type) {
	case string:
		b.UnmarshalHex([]byte(src.(string)))
	default:
		return errors.New("Incompatible type for UUID")
	}
	return nil
}

// Value implements the driver Valuer interface.
func (b Bytes16) Value() (driver.Value, error) {
	return b.MarshalHex()
}
