package iface

import (
	"bytes"
	"database/sql/driver"
	"encoding/hex"
	"errors"
	"fmt"
)

func (b Bytes32) Marshal() ([]byte, error) {
	return b[:], nil
}

func (b Bytes32) MarshalTo(buf []byte) (n int, err error) {
	copy(buf, b[:])
	return len(b), nil
}
func (b *Bytes32) Unmarshal(buf []byte) error {
	if len(buf) != Bytes32Length {
		return fmt.Errorf("invalid bytes32 (got %d bytes)", len(buf))
	}
	copy(b[:], buf)
	return nil
}

func (b *Bytes32) UnmarshalHex(buf []byte) (err error) {
	hexLen := hex.DecodedLen(len(buf))
	if hexLen != Bytes32Length {
		return fmt.Errorf("invalid bytes32 (got %d bytes)", len(buf))

	}
	_, err = hex.Decode(b[:], buf)
	return
}

func (b Bytes32) MarshalHex() ([]byte, error) {
	hexLen := hex.EncodedLen(Bytes32Length)
	hexBuf := make([]byte,hexLen)
	if encodedLen := hex.Encode(hexBuf, b[:]); encodedLen != hexLen {
		return nil, fmt.Errorf("invalid bytes32 length: %d", len(hexBuf))
	}
	return hexBuf, nil
}

func (b Bytes32) MarshalJSON() (encoded []byte, err error) {
	encoded, err = b.MarshalHex()
	if err != nil {
		return
	}
	var buffer bytes.Buffer
	buffer.WriteRune('"')
	buffer.Write(encoded)
	buffer.WriteRune('"')
	return buffer.Bytes(), nil
}
func (b *Bytes32) UnmarshalJSON(from []byte) error {

	quote:=[]byte("\"")
	quoteSize:=len(quote)

	if len(from) < quoteSize*2{
		return errors.New("invalid quote notation")
	}

	if !bytes.HasPrefix(from,quote) ||!bytes.HasSuffix(from,quote){
		return errors.New("invalid quote notation")
	}
	return b.UnmarshalHex(from[quoteSize:len(from)-quoteSize])
}

func (b Bytes32) Compare(other Bytes32) int {
	return bytes.Compare(b[:], other[:])
}

func (b Bytes32) Equal(other Bytes32) bool {
	return b.Compare(other) == 0
}

func (b *Bytes32) Size() int {
	if b == nil {
		return 0
	}
	return Bytes32Length
}

// Scan implements the Scanner interface.
func (b *Bytes32) Scan(src interface{}) error {
	switch src.(type) {
	case string:
		b.UnmarshalHex([]byte(src.(string)))
	default:
		return errors.New("Incompatible type for UUID")
	}
	return nil
}

// Value implements the driver Valuer interface.
func (b Bytes32) Value() (driver.Value, error) {
	return b.MarshalHex()
}
