package util

import (
	"io"
	"os"
	"path"

	"github.com/sirupsen/logrus"
	prefixed "github.com/x-cray/logrus-prefixed-formatter"
	lumberjack "gopkg.in/natefinch/lumberjack.v2"
)

var (
	logger    = logrus.StandardLogger()
	rotaters  []*lumberjack.Logger
	logDir    string
	formatter = prefixed.TextFormatter{}
)

func init() {
	logger.Formatter = &formatter
}
func LoggerIsStd() bool {
	return !formatter.DisableColors
}
func InitLogger(verbose bool, logDirArg string, config *LogConfig) {
	logDir = logDirArg
	colorSupport, writer := NewLogWriter(config)
	formatter.DisableColors = !colorSupport
	logger.Out = writer
	if verbose {
		logger.Level = logrus.DebugLevel
	} else {
		logger.Level = logrus.InfoLevel
	}
}

func NewLogger(prefix string) *logrus.Entry {
	return logrus.NewEntry(logger).WithField("prefix", prefix)
}
func NewLogWriter(config *LogConfig) (bool, io.Writer) {
	switch config.FileName {
	case "Stdout":
		return true, os.Stdout
	case "Stderr":
		return true, os.Stderr
	default:
		logpath := config.FileName
		if logDir != "" {
			logpath = path.Join(logDir, config.FileName)
		}
		logger.Info(" Attention!! log writes to ", config.FileName)
		rotater := &lumberjack.Logger{
			Filename:   logpath,
			MaxSize:    config.MaxSizeMb, // megabytes
			MaxBackups: config.MaxBackup,
			MaxAge:     config.MaxDay, //days
		}
		rotaters = append(rotaters, rotater)
		return false, rotater
	}
}

func RotateLogger() {
	if len(rotaters) == 0 {
		return
	}
	logger.Info("rotating logger")
	for _, rotater := range rotaters {
		rotater.Rotate()
	}
	logger.Info("rotated")
}
